'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _lodash = require('lodash');

var _testRunTracker = require('./test-run-tracker');

var _testRunTracker2 = _interopRequireDefault(_testRunTracker);

var _factorySymbol = require('./factory-symbol');

var _factorySymbol2 = _interopRequireDefault(_factorySymbol);

var _replicator = require('./replicator');

var _observation = require('../test-run/commands/observation');

var _testRun = require('../test-run');

var _testRun2 = _interopRequireDefault(_testRun);

var _compileClientFunction = require('../compiler/es-next/compile-client-function');

var _compileClientFunction2 = _interopRequireDefault(_compileClientFunction);

var _runtime = require('../errors/runtime');

var _message = require('../errors/runtime/message');

var _message2 = _interopRequireDefault(_message);

var _getCallsite = require('../errors/get-callsite');

var _getCallsite2 = _interopRequireDefault(_getCallsite);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DEFAULT_EXECUTION_CALLSITE_NAME = '__$$clientFunction$$';

var ClientFunctionFactory = function () {
    function ClientFunctionFactory(fn, scopeVars) {
        var callsiteNames = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
        (0, _classCallCheck3.default)(this, ClientFunctionFactory);

        this.callsiteNames = {
            instantiation: callsiteNames.instantiation,
            execution: callsiteNames.execution || DEFAULT_EXECUTION_CALLSITE_NAME
        };

        this.functionDescriptor = this._createFunctionDescriptor(fn, scopeVars);

        if (!this.functionDescriptor) throw this._createInvalidFnTypeError(fn);

        this.replicator = (0, _replicator.createReplicator)(this._getReplicatorTransforms());
    }

    ClientFunctionFactory.prototype._validateScopeVars = function _validateScopeVars(scopeVars) {
        var scopeVarsType = typeof scopeVars === 'undefined' ? 'undefined' : (0, _typeof3.default)(scopeVars);

        if (scopeVarsType !== 'object' && scopeVarsType !== 'undefined') throw new _runtime.ClientFunctionAPIError(this.callsiteNames.instantiation, this.callsiteNames.instantiation, _message2.default.clientFunctionScopeVarsIsNotAnObject, scopeVarsType);
    };

    ClientFunctionFactory.prototype._resolveContextTestRun = function _resolveContextTestRun() {
        var testRunId = _testRunTracker2.default.getContextTestRunId();
        var testRun = _testRun2.default.activeTestRuns[testRunId];

        if (!testRun) throw new _runtime.ClientFunctionAPIError(this.callsiteNames.execution, this.callsiteNames.instantiation, _message2.default.clientFunctionCantResolveTestRun);

        return testRun;
    };

    ClientFunctionFactory.prototype._decorateFunction = function _decorateFunction(clientFn) {
        var _this = this;

        clientFn[_factorySymbol2.default] = this;

        clientFn.with = function (options) {
            _this._validateOptions(options);

            return _this.getFunction(options);
        };
    };

    ClientFunctionFactory.prototype.getFunction = function getFunction(options) {
        var factory = this;

        options = options || {};

        var clientFn = function __$$clientFunction$$() {
            var testRun = options.boundTestRun || factory._resolveContextTestRun();
            var callsite = (0, _getCallsite2.default)(factory.callsiteNames.execution);
            var args = [];

            // OPTIMIZATION: don't leak `arguments` object.
            for (var i = 0; i < arguments.length; i++) {
                args.push(arguments[i]);
            }return factory._executeCommand(args, testRun, callsite, options);
        };

        this._decorateFunction(clientFn);

        return clientFn;
    };

    ClientFunctionFactory.prototype.getCommand = function getCommand(args, options) {
        var encodedArgs = this.replicator.encode(args);
        var encodedScopeVars = this.replicator.encode(this.functionDescriptor.scopeVars);

        return this._createExecutionTestRunCommand(encodedArgs, encodedScopeVars, options);
    };

    // Overridable methods


    ClientFunctionFactory.prototype._createFunctionDescriptor = function _createFunctionDescriptor(fn, scopeVars) {
        if (typeof fn === 'function') {
            this._validateScopeVars(scopeVars);

            scopeVars = scopeVars || {};

            return {
                scopeVars: scopeVars,
                fnCode: (0, _compileClientFunction2.default)(fn.toString(), scopeVars, this.callsiteNames.instantiation, this.callsiteNames.instantiation)
            };
        }

        return null;
    };

    ClientFunctionFactory.prototype._createInvalidFnTypeError = function _createInvalidFnTypeError(fn) {
        return new _runtime.ClientFunctionAPIError(this.callsiteNames.instantiation, this.callsiteNames.instantiation, _message2.default.clientFunctionCodeIsNotAFunction, typeof fn === 'undefined' ? 'undefined' : (0, _typeof3.default)(fn));
    };

    ClientFunctionFactory.prototype._executeCommand = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(args, testRun, callsite, options) {
            var command, result;
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            command = this.getCommand(args, options);
                            _context.next = 3;
                            return testRun.executeCommand(command, callsite);

                        case 3:
                            result = _context.sent;
                            return _context.abrupt('return', this.replicator.decode(result));

                        case 5:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this);
        }));

        function _executeCommand(_x2, _x3, _x4, _x5) {
            return ref.apply(this, arguments);
        }

        return _executeCommand;
    }();

    ClientFunctionFactory.prototype._validateOptions = function _validateOptions(options) {
        var optionsType = typeof options === 'undefined' ? 'undefined' : (0, _typeof3.default)(options);

        if (optionsType !== 'object') throw new _runtime.APIError('with', _message2.default.optionsArgumentIsNotAnObject, optionsType);

        if (!(0, _lodash.isNil)(options.boundTestRun)) {
            // NOTE: we can't use strict `t instanceof TestController`
            // check due to module circular reference
            if (!(options.boundTestRun.testRun instanceof _testRun2.default)) throw new _runtime.APIError('with', _message2.default.invalidClientFunctionTestRunBinding);

            // NOTE: boundTestRun is actually a TestController, so we need to unpack it
            options.boundTestRun = options.boundTestRun.testRun;
        }
    };

    ClientFunctionFactory.prototype._createExecutionTestRunCommand = function _createExecutionTestRunCommand(encodedArgs, encodedScopeVars) {
        return new _observation.ExecuteClientFunctionCommand({
            instantiationCallsiteName: this.callsiteNames.instantiation,
            fnCode: this.functionDescriptor.fnCode,
            args: encodedArgs,
            scopeVars: encodedScopeVars
        });
    };

    ClientFunctionFactory.prototype._getReplicatorTransforms = function _getReplicatorTransforms() {
        return [new _replicator.FunctionTransform(this.callsiteNames)];
    };

    return ClientFunctionFactory;
}();

exports.default = ClientFunctionFactory;
module.exports = exports['default'];