'use strict';

exports.__esModule = true;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _lodash = require('lodash');

var _clientFunctionFactory = require('./client-function-factory');

var _clientFunctionFactory2 = _interopRequireDefault(_clientFunctionFactory);

var _replicator = require('./replicator');

var _runtime = require('../errors/runtime');

var _factorySymbol = require('./factory-symbol');

var _factorySymbol2 = _interopRequireDefault(_factorySymbol);

var _message = require('../errors/runtime/message');

var _message2 = _interopRequireDefault(_message);

var _observation = require('../test-run/commands/observation');

var _defineLazyProperty = require('../utils/define-lazy-property');

var _defineLazyProperty2 = _interopRequireDefault(_defineLazyProperty);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var SelectorFactory = function (_ClientFunctionFactor) {
    (0, _inherits3.default)(SelectorFactory, _ClientFunctionFactor);

    function SelectorFactory(fn, scopeVars, callsiteNames) {
        (0, _classCallCheck3.default)(this, SelectorFactory);
        return (0, _possibleConstructorReturn3.default)(this, _ClientFunctionFactor.call(this, fn, scopeVars, callsiteNames));
    }

    SelectorFactory._defineNodeSnapshotDerivativeSelectorProperty = function _defineNodeSnapshotDerivativeSelectorProperty(obj, propName, fn) {
        (0, _defineLazyProperty2.default)(obj, propName, function () {
            var factory = new SelectorFactory(function (fnArg) {
                /* eslint-disable no-undef */
                var selectorResult = selector();

                if (selectorResult && typeof selectorResult.then === 'function') return selectorResult.then(function (node) {
                    return fn(node, fnArg);
                });

                return fn(selectorResult, fnArg);
                /* eslint-enable no-undef */
            }, { selector: obj.selector, fn: fn });

            return factory.getFunction();
        });
    };

    SelectorFactory.prototype._createFunctionDescriptor = function _createFunctionDescriptor(fn, scopeVars) {
        var factoryFromSelector = fn && fn[_factorySymbol2.default];
        var factoryFromPromiseOrSnapshot = fn && fn.selector && fn.selector[_factorySymbol2.default];
        var factory = factoryFromSelector || factoryFromPromiseOrSnapshot;

        if (factory instanceof SelectorFactory) return factory.functionDescriptor;

        if (typeof fn === 'string') {
            return {
                scopeVars: {},
                fnCode: '(function(){return document.querySelector(\'' + fn.replace(/'/g, "\\'") + '\');})'
            };
        }

        return _ClientFunctionFactor.prototype._createFunctionDescriptor.call(this, fn, scopeVars);
    };

    SelectorFactory.prototype._createInvalidFnTypeError = function _createInvalidFnTypeError(fn) {
        return new _runtime.ClientFunctionAPIError(this.callsiteNames.instantiation, this.callsiteNames.instantiation, _message2.default.selectorInitializedWithWrongType, typeof fn === 'undefined' ? 'undefined' : (0, _typeof3.default)(fn));
    };

    SelectorFactory.prototype._executeCommand = function _executeCommand(args, testRun, callsite, options) {
        var _this2 = this;

        var lazyPromise = _promise2.default.resolve();
        var resultPromise = null;

        // OPTIMIZATION: Selectors are executed lazily (once someone subscribes to their result).
        // It's especially useful for situations when the selector's result is passed to
        // an action, e.g.: `t.click(someSelector(42));` In that case, we create a new selector
        // inside this action, but we are not interested in the result of ongoing execution.
        var execute = function execute() {
            if (!resultPromise) {
                resultPromise = _ClientFunctionFactor.prototype._executeCommand.call(_this2, args, testRun, callsite, options).then(function (result) {
                    return result ? _this2._decorateFunctionResult(result, args) : result;
                });
            }

            return resultPromise;
        };

        lazyPromise.then = function (onFulfilled, onRejected) {
            return execute().then(onFulfilled, onRejected);
        };
        lazyPromise.catch = function (onRejected) {
            return execute().catch(onRejected);
        };

        this._defineSelectorPropertyWithBoundArgs(lazyPromise, args);

        return lazyPromise;
    };

    SelectorFactory.prototype._createExecutionTestRunCommand = function _createExecutionTestRunCommand(encodedArgs, encodedScopeVars, options) {
        return new _observation.ExecuteSelectorCommand({
            instantiationCallsiteName: this.callsiteNames.instantiation,
            fnCode: this.functionDescriptor.fnCode,
            args: encodedArgs,
            scopeVars: encodedScopeVars,
            visibilityCheck: !!options.visibilityCheck,
            timeout: options.timeout
        });
    };

    SelectorFactory.prototype._validateOptions = function _validateOptions(options) {
        _ClientFunctionFactor.prototype._validateOptions.call(this, options);

        var visibilityCheckOptionType = (0, _typeof3.default)(options.visibilityCheck);

        if (visibilityCheckOptionType !== 'undefined' && visibilityCheckOptionType !== 'boolean') throw new _runtime.APIError('with', _message2.default.optionValueIsNotABoolean, 'visibilityCheck', visibilityCheckOptionType);

        var timeoutType = (0, _typeof3.default)(options.timeout);
        var isNonNegativeNumber = (0, _lodash.isFinite)(options.timeout) && options.timeout >= 0;

        if (timeoutType !== 'undefined' && !isNonNegativeNumber) {
            var actual = timeoutType === 'number' ? options.timeout : timeoutType;

            throw new _runtime.APIError('with', _message2.default.optionValueIsNotANonNegativeNumber, 'timeout', actual);
        }
    };

    SelectorFactory.prototype._getReplicatorTransforms = function _getReplicatorTransforms() {
        var transforms = _ClientFunctionFactor.prototype._getReplicatorTransforms.call(this);

        transforms.push(new _replicator.SelectorNodeTransform());

        return transforms;
    };

    SelectorFactory.prototype._defineSelectorPropertyWithBoundArgs = function _defineSelectorPropertyWithBoundArgs(obj, selectorArgs) {
        var _this3 = this;

        (0, _defineLazyProperty2.default)(obj, 'selector', function () {
            var factory = new SelectorFactory(function () {
                return (/* eslint-disable no-undef */selector.apply(null, args)
                );
            } /* eslint-enable no-undef */, {
                selector: _this3.getFunction(),
                args: selectorArgs
            });

            return factory.getFunction();
        });
    };

    SelectorFactory.prototype._decorateFunctionResult = function _decorateFunctionResult(nodeSnapshot, selectorArgs) {
        this._defineSelectorPropertyWithBoundArgs(nodeSnapshot, selectorArgs);

        SelectorFactory._defineNodeSnapshotDerivativeSelectorProperty(nodeSnapshot, 'getParentNode', function (node) {
            return node ? node.parentNode : node;
        });

        SelectorFactory._defineNodeSnapshotDerivativeSelectorProperty(nodeSnapshot, 'getChildNode', function (node, idx) {
            return node ? node.childNodes[idx] : node;
        });

        SelectorFactory._defineNodeSnapshotDerivativeSelectorProperty(nodeSnapshot, 'getChildElement', function (node, idx) {
            if (node.children) return node.children[idx];

            // NOTE: IE doesn't have `children` for non-element nodes =/
            var childNodeCount = node.childNodes.length;
            var currentElIdx = 0;

            for (var i = 0; i < childNodeCount; i++) {
                if (node.childNodes[i].nodeType === 1) {
                    if (currentElIdx === idx) return node.childNodes[i];

                    currentElIdx++;
                }
            }

            return null;
        });

        if (nodeSnapshot.classNames) nodeSnapshot.hasClass = function (name) {
            return nodeSnapshot.classNames.indexOf(name) > -1;
        };

        return nodeSnapshot;
    };

    return SelectorFactory;
}(_clientFunctionFactory2.default);

exports.default = SelectorFactory;
module.exports = exports['default'];