'use strict';

exports.__esModule = true;

var _TYPE$okAssertion$TYP;

var _dedent = require('dedent');

var _dedent2 = _interopRequireDefault(_dedent);

var _lodash = require('lodash');

var _type = require('./type');

var _type2 = _interopRequireDefault(_type);

var _assertionDiffs = require('../../errors/test-run/assertion-diffs');

var _assertionDiffs2 = _interopRequireDefault(_assertionDiffs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function escapeNewLines(str) {
    return (0, _lodash.escape)(str).replace(/(\r\n|\n|\r)/gm, '\\n');
}

function getMsgPrefix(err) {
    return '<span class="user-agent">' + err.userAgent + '</span>\n';
}

function getAssertionMsgPrefix(err) {
    var assertionPrefix = err.message ? '"' + (0, _lodash.escape)(err.message) + '" assertion' : 'Assertion';

    return getMsgPrefix(err) + assertionPrefix;
}

function getDiffHeader(err) {
    if (err.isArrays) return 'Arrays differ at index <code>' + err.key + '</code>:';

    if (err.isObjects) return 'Objects differ at the <code>' + (0, _lodash.escape)(err.key) + '</code> field:';

    if (err.isStrings) return 'Strings differ at index <code>' + err.key + '</code>:';

    return '';
}

function getScreenshotInfoStr(err) {
    if (err.screenshotPath) return '<div class="screenshot-info"><strong>Screenshot:</strong> <a class="screenshot-path">' + (0, _lodash.escape)(err.screenshotPath) + '</a></div>';

    return '';
}

exports.default = (_TYPE$okAssertion$TYP = {}, _TYPE$okAssertion$TYP[_type2.default.okAssertion] = function (err) {
    return (0, _dedent2.default)('\n        ' + getAssertionMsgPrefix(err) + ' failed at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        <strong>Expected: </strong>not <code>null</code>, not <code>undefined</code>, not <code>false</code>, not <code>NaN</code> and not <code>\'\'</code>\n        <strong>Actual:   </strong><code>' + escapeNewLines(err.actual) + '</code>\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.notOkAssertion] = function (err) {
    return (0, _dedent2.default)('\n        ' + getAssertionMsgPrefix(err) + ' failed at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        <strong>Expected: </strong><code>null</code>, <code>undefined</code>, <code>false</code>, <code>NaN</code> or <code>\'\'</code>\n        <strong>Actual:   </strong><code>' + escapeNewLines(err.actual) + '</code>\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.eqAssertion] = function (err, maxStringLength) {
    var diff = (0, _assertionDiffs2.default)(err, maxStringLength);
    var diffMarkerStr = diff.marker ? '          ' + diff.marker : '';

    return (0, _dedent2.default)('\n            ' + getAssertionMsgPrefix(err) + ' failed at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n            ' + err.getCallsiteMarkup() + '\n\n            ' + getDiffHeader(err) + '\n\n            <strong>Expected: </strong><code>' + escapeNewLines(diff.expected) + '</code>\n            <strong>Actual:   </strong><code>' + escapeNewLines(diff.actual) + '</code>\n            <code>' + diffMarkerStr + '</code>\n\n            ' + getScreenshotInfoStr(err) + '\n        ');
}, _TYPE$okAssertion$TYP[_type2.default.notEqAssertion] = function (err) {
    return (0, _dedent2.default)('\n        ' + getAssertionMsgPrefix(err) + ' failed at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        <strong>Expected: </strong>not <code>' + escapeNewLines(err.actual) + '</code>\n        <strong>Actual:   </strong><code>' + escapeNewLines(err.actual) + '</code>\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.iframeLoadingTimeout] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'IFrame loading timed out.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.inIFrameTargetLoadingTimeout] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n        IFrame target loading timed out.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.uncaughtJSError] = function (err) {
    if (err.pageDestUrl) {
        return (0, _dedent2.default)('\n                ' + getMsgPrefix(err) + 'Uncaught JavaScript error <code>' + (0, _lodash.escape)(err.scriptErr) + '</code> on page <a href="' + err.pageDestUrl + '">' + err.pageDestUrl + '</a>\n\n                ' + getScreenshotInfoStr(err) + '\n            ');
    }

    return (0, _dedent2.default)('\n            ' + getMsgPrefix(err) + 'Uncaught JavaScript error <code>' + (0, _lodash.escape)(err.scriptErr) + '</code> on page.\n\n            ' + getScreenshotInfoStr(err) + '\n        ');
}, _TYPE$okAssertion$TYP[_type2.default.uncaughtJSErrorInTestCodeStep] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n        Uncaught JavaScript error in test code - <code>' + (0, _lodash.escape)(err.scriptErr) + '</code>.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.storeDomNodeOrJqueryObject] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n        It is not allowed to share the DOM element, jQuery object or a function between test steps via "this" object.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.emptyFirstArgument] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        A target element of the <code class="api">' + err.action + '</code> action has not been found in the DOM tree.\n        If this element should be created after animation or a time-consuming operation is finished, use the <code class="api">waitFor</code> action (available for use in code) to pause test execution until this element appears.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.invisibleActionElement] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        A target element <code>' + (0, _lodash.escape)(err.element) + '</code> of the <code class="api">' + err.action + '</code> action is not visible.\n        If this element should appear when you are hovering over another element, make sure that you properly recorded the <code class="api">hover</code> action.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.incorrectDraggingSecondArgument] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        <code class="api">drag</code> action drop target is incorrect.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.incorrectPressActionArgument] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        <code class="api">press</code> action parameter contains incorrect key code.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.emptyTypeActionArgument] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        The <code class="api">type<code> action\'s parameter text is empty.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.unexpectedDialog] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n        Unexpected system <code>' + err.dialog + '</code> dialog <code>' + (0, _lodash.escape)(err.message) + '</code> appeared.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.expectedDialogDoesntAppear] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n        The expected system <code>' + err.dialog + '</code> dialog did not appear.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.incorrectSelectActionArguments] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        <code class="api">select</code> action\'s parameters contain an incorrect value.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.incorrectWaitActionMillisecondsArgument] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        <code class="api">wait</code> action\'s "milliseconds" parameter should be a positive number.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.incorrectWaitForActionEventArgument] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        <code class="api">waitFor</code> action\'s first parameter should be a function, a CSS selector or an array of CSS selectors.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.incorrectWaitForActionTimeoutArgument] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        <code class="api">waitFor</code> action\'s "timeout" parameter should be a positive number.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.waitForActionTimeoutExceeded] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        <code class="api">waitFor</code> action\'s timeout exceeded.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.incorrectGlobalWaitForActionEventArgument] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        <code class="api">__waitFor</code> action\'s first parameter should be a function.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.incorrectGlobalWaitForActionTimeoutArgument] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        <code class="api">__waitFor</code> action\'s "timeout" parameter should be a positive number.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.globalWaitForActionTimeoutExceeded] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        <code class="api">__waitFor</code> action\'s timeout exceeded.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.emptyIFrameArgument] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n        The selector within the <code class="api">inIFrame</code> function returns an empty value.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.iframeArgumentIsNotIFrame] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n        The selector within the <code class="api">inIFrame</code> function doesn’t return an iframe element.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.multipleIFrameArgument] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n        The selector within the <code class="api">inIFrame</code> function returns more than one iframe element.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.incorrectIFrameArgument] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n        The <code class="api">inIFrame</code> function contains an invalid argument.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.uploadCanNotFindFileToUpload] = function (err) {
    var msg = (0, _dedent2.default)('\n            ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n            ' + err.getCallsiteMarkup() + '\n\n            Cannot find the following file(s) to upload:\n        ');

    return msg + err.filePaths.map(function (path) {
        return '\n    <code>' + (0, _lodash.escape)(path) + '</code>';
    }).join(',') + (err.screenshotPath ? '\n\n' + getScreenshotInfoStr(err) : '');
}, _TYPE$okAssertion$TYP[_type2.default.uploadElementIsNotFileInput] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        <code class="api">upload</code> action argument does not contain a file input element.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.uploadInvalidFilePathArgument] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + 'Error at step <span class="step-name">' + (0, _lodash.escape)(err.stepName) + '</span>:\n\n        ' + err.getCallsiteMarkup() + '\n\n        <code class="api">upload</code> action\'s "path" parameter should be a string or an array of strings.\n\n        ' + getScreenshotInfoStr(err) + '\n    ');
}, _TYPE$okAssertion$TYP[_type2.default.pageNotLoaded] = function (err) {
    return (0, _dedent2.default)('\n        ' + getMsgPrefix(err) + err.message + '\n    ');
}, _TYPE$okAssertion$TYP);
module.exports = exports['default'];