'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _events = require('events');

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _timeLimitPromise = require('time-limit-promise');

var _timeLimitPromise2 = _interopRequireDefault(_timeLimitPromise);

var _promisifyEvent = require('promisify-event');

var _promisifyEvent2 = _interopRequireDefault(_promisifyEvent);

var _lodash = require('lodash');

var _mapReverse = require('map-reverse');

var _mapReverse2 = _interopRequireDefault(_mapReverse);

var _local = require('../browser-connection/local');

var _local2 = _interopRequireDefault(_local);

var _runtime = require('../errors/runtime');

var _message = require('../errors/runtime/message');

var _message2 = _interopRequireDefault(_message);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var BrowserSet = function (_EventEmitter) {
    (0, _inherits3.default)(BrowserSet, _EventEmitter);

    function BrowserSet(connections) {
        (0, _classCallCheck3.default)(this, BrowserSet);

        var _this = (0, _possibleConstructorReturn3.default)(this, _EventEmitter.call(this));

        _this.READY_TIMEOUT = 30000;
        _this.RELEASE_TIMEOUT = 10000;

        _this.pendingReleases = [];

        _this.connections = connections;

        _this.browserErrorHandler = function (error) {
            return _this.emit('error', error);
        };

        connections.forEach(function (bc) {
            return bc.on('error', _this.browserErrorHandler);
        });

        // NOTE: We're setting an empty error handler, because Node kills the process on an 'error' event
        // if there is no handler. See: https://nodejs.org/api/events.html#events_class_events_eventemitter
        _this.on('error', _lodash.noop);
        return _this;
    }

    BrowserSet._waitIdle = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(bc) {
            var idlePromise, closePromise;
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            if (!(bc.idle || bc.closed || !bc.ready)) {
                                _context.next = 2;
                                break;
                            }

                            return _context.abrupt('return');

                        case 2:
                            idlePromise = (0, _promisifyEvent2.default)(bc, 'idle');
                            closePromise = (0, _promisifyEvent2.default)(bc, 'closed');
                            _context.next = 6;
                            return _pinkie2.default.race([idlePromise, closePromise]);

                        case 6:

                            // NOTE: We must delete both listeners from the browser connection after
                            // one of them is executed to avoid exceeding the subscribers limit.
                            idlePromise.cancel();
                            closePromise.cancel();

                        case 8:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this);
        }));

        function _waitIdle(_x) {
            return ref.apply(this, arguments);
        }

        return _waitIdle;
    }();

    BrowserSet._closeConnection = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(bc) {
            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            if (!(bc.closed || !bc.ready)) {
                                _context2.next = 2;
                                break;
                            }

                            return _context2.abrupt('return');

                        case 2:

                            bc.close();

                            _context2.next = 5;
                            return (0, _promisifyEvent2.default)(bc, 'closed');

                        case 5:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, this);
        }));

        function _closeConnection(_x2) {
            return ref.apply(this, arguments);
        }

        return _closeConnection;
    }();

    BrowserSet.prototype._waitConnectionsReady = function _waitConnectionsReady() {
        var connectionsReadyPromise = _pinkie2.default.all(this.connections.filter(function (bc) {
            return !bc.ready;
        }).map(function (bc) {
            return (0, _promisifyEvent2.default)(bc, 'ready');
        }));

        var timeoutError = new _runtime.GeneralError(_message2.default.cantEstablishBrowserConnection);

        return (0, _timeLimitPromise2.default)(connectionsReadyPromise, this.READY_TIMEOUT, { rejectWith: timeoutError });
    };

    BrowserSet.prototype._checkForDisconnections = function _checkForDisconnections() {
        var disconnectedUserAgents = this.connections.filter(function (bc) {
            return bc.closed;
        }).map(function (bc) {
            return bc.userAgent;
        });

        if (disconnectedUserAgents.length) throw new _runtime.GeneralError(_message2.default.cantRunAgainstDisconnectedBrowsers, disconnectedUserAgents.join(', '));
    };

    //API


    BrowserSet.from = function from(connections) {
        var _this2 = this;

        var browserSet = new BrowserSet(connections);

        var prepareConnection = _pinkie2.default.resolve().then(function () {
            browserSet._checkForDisconnections();
            return browserSet._waitConnectionsReady();
        }).then(function () {
            return browserSet;
        });

        return _pinkie2.default.race([prepareConnection, (0, _promisifyEvent2.default)(browserSet, 'error')]).catch(function () {
            var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(error) {
                return _regenerator2.default.wrap(function _callee3$(_context3) {
                    while (1) {
                        switch (_context3.prev = _context3.next) {
                            case 0:
                                _context3.next = 2;
                                return browserSet.dispose();

                            case 2:
                                throw error;

                            case 3:
                            case 'end':
                                return _context3.stop();
                        }
                    }
                }, _callee3, _this2);
            }));
            return function (_x3) {
                return ref.apply(this, arguments);
            };
        }());
    };

    BrowserSet.prototype.releaseConnection = function releaseConnection(bc) {
        var _this3 = this;

        if (this.connections.indexOf(bc) < 0) return _pinkie2.default.resolve();

        (0, _lodash.pull)(this.connections, bc);

        bc.removeListener('error', this.browserErrorHandler);

        var appropriateStateSwitch = bc instanceof _local2.default ? BrowserSet._closeConnection(bc) : BrowserSet._waitIdle(bc);

        var release = (0, _timeLimitPromise2.default)(appropriateStateSwitch, this.RELEASE_TIMEOUT).then(function () {
            return (0, _lodash.pull)(_this3.pendingReleases, release);
        });

        this.pendingReleases.push(release);

        return release;
    };

    BrowserSet.prototype.dispose = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
            var _this4 = this;

            return _regenerator2.default.wrap(function _callee4$(_context4) {
                while (1) {
                    switch (_context4.prev = _context4.next) {
                        case 0:
                            // NOTE: When browserConnection is cancelled, it is removed from
                            // the this.connections array, which leads to shifting indexes
                            // towards the beginning. So, we must copy the array in order to iterate it,
                            // or we can perform iteration from the end to the beginning.
                            (0, _mapReverse2.default)(this.connections, function (bc) {
                                return _this4.releaseConnection(bc);
                            });

                            _context4.next = 3;
                            return _pinkie2.default.all(this.pendingReleases);

                        case 3:
                        case 'end':
                            return _context4.stop();
                    }
                }
            }, _callee4, this);
        }));

        function dispose() {
            return ref.apply(this, arguments);
        }

        return dispose;
    }();

    return BrowserSet;
}(_events.EventEmitter);

exports.default = BrowserSet;
module.exports = exports['default'];