'use strict';

exports.__esModule = true;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _path = require('path');

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

var _lodash = require('lodash');

var _capturer = require('./capturer');

var _capturer2 = _interopRequireDefault(_capturer);

var _warningMessages = require('./warning-messages');

var _warningMessages2 = _interopRequireDefault(_warningMessages);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Screenshots = function () {
    function Screenshots(path) {
        (0, _classCallCheck3.default)(this, Screenshots);

        this.enabled = !!path;
        this.screenshotsPath = path;
        this.testEntries = [];
    }

    Screenshots._escapeUserAgent = function _escapeUserAgent(userAgent) {
        return userAgent.toString().split('/').map(function (str) {
            return str.trim().replace(/\s/g, '_');
        }).join('_');
    };

    Screenshots.prototype._addTestEntry = function _addTestEntry(test) {
        var testEntry = {
            test: test,
            path: this.screenshotsPath ? (0, _path.join)(this.screenshotsPath, _uuid2.default.v4().substr(0, 8)) : '',
            screenshotCapturingCalled: false
        };

        this.testEntries.push(testEntry);

        return testEntry;
    };

    Screenshots.prototype._getTestEntry = function _getTestEntry(test) {
        return (0, _lodash.find)(this.testEntries, function (entry) {
            return entry.test === test;
        });
    };

    Screenshots.prototype.hasCapturedFor = function hasCapturedFor(test) {
        return this._getTestEntry(test).screenshotCapturingCalled;
    };

    Screenshots.prototype.getPathFor = function getPathFor(test) {
        if (this._getTestEntry(test).screenshotCapturingCalled && !this.enabled) return _warningMessages2.default.screenshotDirNotSet;

        return this._getTestEntry(test).path;
    };

    Screenshots.prototype.createCapturerFor = function createCapturerFor(test, userAgent) {
        var testEntry = this._getTestEntry(test);

        if (!testEntry) testEntry = this._addTestEntry(test);

        var testScreenshotsPath = (0, _path.join)(testEntry.path, Screenshots._escapeUserAgent(userAgent));

        return new _capturer2.default(this.screenshotsPath, testScreenshotsPath, testEntry);
    };

    return Screenshots;
}();

exports.default = Screenshots;
module.exports = exports['default'];