'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _readFileRelative = require('read-file-relative');

var _pinkie = require('pinkie');

var _pinkie2 = _interopRequireDefault(_pinkie);

var _mustache = require('mustache');

var _mustache2 = _interopRequireDefault(_mustache);

var _testcafeHammerhead = require('testcafe-hammerhead');

var _debugLog = require('./debug-log');

var _debugLog2 = _interopRequireDefault(_debugLog);

var _formattableAdapter = require('../errors/test-run/formattable-adapter');

var _formattableAdapter2 = _interopRequireDefault(_formattableAdapter);

var _testRun = require('../errors/test-run/');

var _browserManipulationManager = require('./browser-manipulation-manager');

var _browserManipulationManager2 = _interopRequireDefault(_browserManipulationManager);

var _clientMessages = require('./client-messages');

var _clientMessages2 = _interopRequireDefault(_clientMessages);

var _state = require('./state');

var _state2 = _interopRequireDefault(_state);

var _type = require('./commands/type');

var _type2 = _interopRequireDefault(_type);

var _windowManipulation = require('./commands/window-manipulation');

var _service = require('./commands/service');

var _utils = require('./commands/utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//Const
var TEST_RUN_TEMPLATE = (0, _readFileRelative.readSync)('../client/test-run/index.js.mustache');
var IFRAME_TEST_RUN_TEMPLATE = (0, _readFileRelative.readSync)('../client/test-run/iframe.js.mustache');
var TEST_DONE_CONFIRMATION_RESPONSE = 'test-done-confirmation';
var MAX_RESPONSE_DELAY = 2 * 60 * 1000;

var TestRun = function (_Session) {
    (0, _inherits3.default)(TestRun, _Session);

    function TestRun(test, browserConnection, screenshotCapturer, opts) {
        (0, _classCallCheck3.default)(this, TestRun);

        var uploadsRoot = _path2.default.dirname(test.fixture.path);

        var _this = (0, _possibleConstructorReturn3.default)(this, _Session.call(this, uploadsRoot));

        _this.opts = opts;
        _this.test = test;
        _this.browserConnection = browserConnection;
        _this.browserManipulationManager = new _browserManipulationManager2.default(screenshotCapturer);

        _this.running = false;
        _this.state = _state2.default.initial;

        _this.driverTaskQueue = [];
        _this.browserManipulationQueue = [];
        _this.testDoneCommandQueued = false;

        _this.pendingRequest = null;
        _this.pendingPageError = null;

        _this.debugLog = new _debugLog2.default(_this.browserConnection.userAgent);

        _this.injectable.scripts.push('/testcafe-core.js');
        _this.injectable.scripts.push('/testcafe-ui.js');
        _this.injectable.scripts.push('/testcafe-runner.js');
        _this.injectable.scripts.push('/testcafe-driver.js');
        _this.injectable.styles.push('/testcafe-ui-styles.css');

        _this.errs = [];
        _this.lastDriverStatusId = null;
        _this.lastDriverStatusResponse = null;
        return _this;
    }

    // Hammerhead payload


    TestRun.prototype._getPayloadScript = function _getPayloadScript() {
        return _mustache2.default.render(TEST_RUN_TEMPLATE, {
            testRunId: this.id,
            browserHeartbeatUrl: this.browserConnection.heartbeatUrl,
            browserStatusUrl: this.browserConnection.statusUrl,
            selectorTimeout: this.opts.selectorTimeout
        });
    };

    TestRun.prototype._getIframePayloadScript = function _getIframePayloadScript() {
        return _mustache2.default.render(IFRAME_TEST_RUN_TEMPLATE, {
            testRunId: this.id,
            selectorTimeout: this.opts.selectorTimeout
        });
    };

    // Hammerhead handlers


    TestRun.prototype.getAuthCredentials = function getAuthCredentials() {
        // TODO
    };

    TestRun.prototype.handleFileDownload = function handleFileDownload() {
        // TODO
    };

    TestRun.prototype.handlePageError = function handlePageError(ctx, err) {
        this.pendingPageError = new _testRun.PageLoadError(err);

        ctx.redirect(ctx.toProxyUrl('about:error'));
    };

    // Test function execution


    TestRun.prototype._executeTestFn = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(state, fn) {
            var screenshotPath;
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            this.state = state;

                            _context.prev = 1;
                            _context.next = 4;
                            return fn(this);

                        case 4:
                            _context.next = 15;
                            break;

                        case 6:
                            _context.prev = 6;
                            _context.t0 = _context['catch'](1);
                            screenshotPath = null;

                            if (!this.opts.takeScreenshotsOnFails) {
                                _context.next = 13;
                                break;
                            }

                            _context.next = 12;
                            return this.executeCommand(new _windowManipulation.TakeScreenshotOnFailCommand());

                        case 12:
                            screenshotPath = _context.sent;

                        case 13:

                            this._addError(_context.t0, screenshotPath);
                            return _context.abrupt('return', false);

                        case 15:
                            return _context.abrupt('return', !this._addPendingPageErrorIfAny());

                        case 16:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this, [[1, 6]]);
        }));

        function _executeTestFn(_x, _x2) {
            return ref.apply(this, arguments);
        }

        return _executeTestFn;
    }();

    TestRun.prototype._start = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
            var beforeEachFn, afterEachFn;
            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            beforeEachFn = this.test.fixture.beforeEachFn;
                            afterEachFn = this.test.fixture.afterEachFn;


                            TestRun.activeTestRuns[this.id] = this;

                            this.running = true;
                            this.emit('start');

                            _context2.t0 = !beforeEachFn;

                            if (_context2.t0) {
                                _context2.next = 10;
                                break;
                            }

                            _context2.next = 9;
                            return this._executeTestFn(_state2.default.inBeforeEach, beforeEachFn);

                        case 9:
                            _context2.t0 = _context2.sent;

                        case 10:
                            if (!_context2.t0) {
                                _context2.next = 16;
                                break;
                            }

                            _context2.next = 13;
                            return this._executeTestFn(_state2.default.inTest, this.test.fn);

                        case 13:
                            if (!afterEachFn) {
                                _context2.next = 16;
                                break;
                            }

                            _context2.next = 16;
                            return this._executeTestFn(_state2.default.inAfterEach, afterEachFn);

                        case 16:
                            _context2.next = 18;
                            return this.executeCommand(new _service.TestDoneCommand());

                        case 18:
                            this._addPendingPageErrorIfAny();

                            delete TestRun.activeTestRuns[this.id];

                            this.emit('done');

                        case 21:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, this);
        }));

        function _start() {
            return ref.apply(this, arguments);
        }

        return _start;
    }();

    // Errors


    TestRun.prototype._addPendingPageErrorIfAny = function _addPendingPageErrorIfAny() {
        if (this.pendingPageError) {
            this._addError(this.pendingPageError);
            this.pendingPageError = null;
            return true;
        }

        return false;
    };

    TestRun.prototype._addError = function _addError(err, screenshotPath) {
        var adapter = new _formattableAdapter2.default(err, {
            userAgent: this.browserConnection.userAgent,
            screenshotPath: screenshotPath || '',
            testRunState: this.state
        });

        this.errs.push(adapter);
    };

    // Task queue


    TestRun.prototype._enqueueCommand = function _enqueueCommand(command, callsite) {
        var _this2 = this;

        if (this.pendingRequest) this._resolvePendingRequest(command);

        return new _pinkie2.default(function (resolve, reject) {
            return _this2.driverTaskQueue.push({ command: command, resolve: resolve, reject: reject, callsite: callsite });
        });
    };

    TestRun.prototype._removeAllNonServiceTasks = function _removeAllNonServiceTasks() {
        this.driverTaskQueue = this.driverTaskQueue.filter(function (driverTask) {
            return (0, _utils.isServiceCommand)(driverTask.command);
        });
        this.browserManipulationQueue = this.browserManipulationQueue.filter(function (manipulationTask) {
            return (0, _utils.isServiceCommand)(manipulationTask.command);
        });
    };

    // Current driver task


    TestRun.prototype._resolveCurrentDriverTask = function _resolveCurrentDriverTask(result) {
        this.currentDriverTask.resolve(result);
        this.driverTaskQueue.shift();

        if (this.testDoneCommandQueued) this._removeAllNonServiceTasks();
    };

    TestRun.prototype._rejectCurrentDriverTask = function _rejectCurrentDriverTask(err) {
        err.callsite = err.callsite || this.driverTaskQueue[0].callsite;

        this.currentDriverTask.reject(err);
        this._removeAllNonServiceTasks();
    };

    // Pending request


    TestRun.prototype._clearPendingRequest = function _clearPendingRequest() {
        if (this.pendingRequest) {
            clearTimeout(this.pendingRequest.responseTimeout);
            this.pendingRequest = null;
        }
    };

    TestRun.prototype._resolvePendingRequest = function _resolvePendingRequest(command) {
        this.lastDriverStatusResponse = command;
        this.pendingRequest.resolve(command);
        this._clearPendingRequest();
    };

    // Handle driver request


    TestRun.prototype._fulfillCurrentDriverTask = function _fulfillCurrentDriverTask(driverStatus) {
        if (driverStatus.executionError) this._rejectCurrentDriverTask(driverStatus.executionError);else this._resolveCurrentDriverTask(driverStatus.result);
    };

    TestRun.prototype._handlePageErrorStatus = function _handlePageErrorStatus(pageError) {
        if (this.currentDriverTask && (0, _utils.isCommandRejectableByPageError)(this.currentDriverTask.command)) {
            this._rejectCurrentDriverTask(pageError);
            this.pendingPageError = null;

            return true;
        }

        this.pendingPageError = this.pendingPageError || pageError;

        return false;
    };

    TestRun.prototype._handleDriverRequest = function _handleDriverRequest(driverStatus) {
        if (!this.running) this._start();

        var pageError = this.pendingPageError || driverStatus.pageError;

        var currentTaskRejectedByError = pageError && this._handlePageErrorStatus(pageError);

        if (!currentTaskRejectedByError && driverStatus.isCommandResult) {
            if (this.currentDriverTask.command.type === _type2.default.testDone) {
                this._resolveCurrentDriverTask();

                return TEST_DONE_CONFIRMATION_RESPONSE;
            }

            this._fulfillCurrentDriverTask(driverStatus);
        }

        return this.currentDriverTask ? this.currentDriverTask.command : null;
    };

    // Execute command


    TestRun.prototype.executeCommand = function executeCommand(command, callsite) {
        this.debugLog.command(command);

        if (this.pendingPageError && (0, _utils.isCommandRejectableByPageError)(command)) return this._rejectCommandWithPageError(callsite);

        if ((0, _utils.isBrowserManipulationCommand)(command)) {
            this.browserManipulationQueue.push(command);

            return this.executeCommand(new _service.PrepareBrowserManipulationCommand(command.type), callsite);
        }

        if (command.type === _type2.default.wait) return new _pinkie2.default(function (resolve) {
            return setTimeout(resolve, command.timeout);
        });

        if (command.type === _type2.default.testDone) this.testDoneCommandQueued = true;

        return this._enqueueCommand(command, callsite);
    };

    TestRun.prototype._rejectCommandWithPageError = function _rejectCommandWithPageError(callsite) {
        var err = this.pendingPageError;

        err.callsite = callsite;
        this.pendingPageError = null;

        return _pinkie2.default.reject(err);
    };

    (0, _createClass3.default)(TestRun, [{
        key: 'currentDriverTask',
        get: function get() {
            return this.driverTaskQueue[0];
        }
    }]);
    return TestRun;
}(_testcafeHammerhead.Session);

// Active test runs pool, used by client functions


exports.default = TestRun;
TestRun.activeTestRuns = {};

// Service message handlers
var ServiceMessages = TestRun.prototype;

ServiceMessages[_clientMessages2.default.ready] = function (msg) {
    var _this3 = this;

    this.debugLog.driverMessage(msg);

    this._clearPendingRequest();

    // NOTE: the driver sends the status for the second time if it didn't get a response at the
    // first try. This is possible when the page was unloaded after the driver sent the status.
    if (msg.status.id === this.lastDriverStatusId) return this.lastDriverStatusResponse;

    this.lastDriverStatusId = msg.status.id;
    this.lastDriverStatusResponse = this._handleDriverRequest(msg.status);

    if (this.lastDriverStatusResponse) return this.lastDriverStatusResponse;

    // NOTE: browsers abort an opened xhr request after a certain timeout (the actual duration depends on the browser).
    // To avoid this, we send an empty response after 2 minutes if we didn't get any command.
    var responseTimeout = setTimeout(function () {
        return _this3._resolvePendingRequest(null);
    }, MAX_RESPONSE_DELAY);

    return new _pinkie2.default(function (resolve, reject) {
        return _this3.pendingRequest = { resolve: resolve, reject: reject, responseTimeout: responseTimeout };
    });
};

ServiceMessages[_clientMessages2.default.readyForBrowserManipulation] = function () {
    var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(msg) {
        var command;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
            while (1) {
                switch (_context3.prev = _context3.next) {
                    case 0:
                        this.debugLog.driverMessage(msg);

                        command = this.browserManipulationQueue.shift();

                        if (!(command.type === _type2.default.takeScreenshot)) {
                            _context3.next = 6;
                            break;
                        }

                        _context3.next = 5;
                        return this.browserManipulationManager.takeScreenshot(this.id, command.path);

                    case 5:
                        return _context3.abrupt('return', _context3.sent);

                    case 6:
                        if (!(command.type === _type2.default.takeScreenshotOnFail)) {
                            _context3.next = 10;
                            break;
                        }

                        _context3.next = 9;
                        return this.browserManipulationManager.takeScreenshotOnFail(this.id);

                    case 9:
                        return _context3.abrupt('return', _context3.sent);

                    case 10:
                        if (!(command.type === _type2.default.resizeWindow)) {
                            _context3.next = 14;
                            break;
                        }

                        _context3.next = 13;
                        return _browserManipulationManager2.default.resizeWindow(this.id, msg.currentWidth, msg.currentHeight, command.width, command.height);

                    case 13:
                        return _context3.abrupt('return', _context3.sent);

                    case 14:
                        if (!(command.type === _type2.default.resizeWindowToFitDevice)) {
                            _context3.next = 18;
                            break;
                        }

                        _context3.next = 17;
                        return _browserManipulationManager2.default.resizeWindowToFitDevice(this.id, msg.currentWidth, msg.currentHeight, command.device, command.options.portraitOrientation);

                    case 17:
                        return _context3.abrupt('return', _context3.sent);

                    case 18:
                    case 'end':
                        return _context3.stop();
                }
            }
        }, _callee3, this);
    }));
    return function (_x3) {
        return ref.apply(this, arguments);
    };
}();
module.exports = exports['default'];